'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var util = require('util');
var stream = require('stream');
var cloneable = _interopDefault(require('cloneable-readable'));
var events = require('events');
var mergeStream = _interopDefault(require('merge-stream'));
var splitStream = _interopDefault(require('split'));
var child_process = require('child_process');
var path = require('path');
var npmRunPath = _interopDefault(require('npm-run-path'));
var which = _interopDefault(require('which'));
var serializeJs = _interopDefault(require('serialize-javascript'));
var chalk = _interopDefault(require('chalk'));
var t = require('ts-interface-checker');
var net = require('net');

// https://github.com/npm/node-which/blob/6a822d836de79f92fb3170f685a6e283fbfeff87/which.js#L1-L3

const isWindows = process.platform === 'win32' || process.env.OSTYPE === 'cygwin' || process.env.OSTYPE === 'msys';
function spawnProcess(config) {
  const cwd = path.resolve(config.cwd);
  let [binary, ...args] = config.command;
  let env = { ...config.env
  };
  let path$1 = readEnvCaseInsensitive(env, 'PATH') || '';
  path$1 = filterBlankParts(npmRunPath({
    cwd,
    path: path$1
  }));
  env = writeEnvCaseNormalized(env, 'PATH', path$1);

  if (isWindows) {
    const pathExt = readEnvCaseInsensitive(env, 'PATHEXT') || '.COM;.EXE;.BAT;.CMD;.VBS;.VBE;.JS;.JSE;.WSF;.WSH';
    env = writeEnvCaseNormalized(env, 'PATHEXT', pathExt);
    /*
      Work around issue (same issue):
        - https://github.com/nodejs/node-v0.x-archive/issues/2318
        - https://github.com/nodejs/node/issues/6671
           Without resorting to wrapping command in shell (like npm package `cross-spawn` does)
      because [spawning a shell is expensive](https://github.com/nodejs/node/issues/6671#issuecomment-219210529)
      and more importantly, that approach introduces *more* window-linux disparities.
           Instead just replace `binary` with a fully-qualified version.
     */

    binary = path.normalize(myWhich(cwd, binary, path$1, pathExt) || binary);
  }

  return child_process.spawn(binary, args, {
    cwd,
    env
  });
}

function readEnvCaseInsensitive(env, key) {
  const upperCaseKey = key.toUpperCase();
  const caseInsensitiveKey = Object.keys(env).reverse().find(key => key.toUpperCase() === upperCaseKey);
  return caseInsensitiveKey === undefined ? undefined : env[caseInsensitiveKey];
}

function writeEnvCaseNormalized(env, key, value) {
  const upperCaseKey = key.toUpperCase();
  return { ...Object.fromEntries(Object.entries(env).filter(([key]) => key.toUpperCase() !== upperCaseKey)),
    [upperCaseKey]: value
  };
}

function filterBlankParts(string) {
  const colon = isWindows ? ';' : ':';
  return string.split(colon).filter(Boolean).join(colon);
} // Version of `which.sync` that adds a `cwd` parameter & doesn't throw


function myWhich(cwd, binary, path, pathExt) {
  const originalCwd = process.cwd();

  if (cwd !== originalCwd) {
    process.chdir(cwd);
  }

  const result = which.sync(binary, {
    nothrow: true,
    path,
    pathExt
  });

  if (cwd !== originalCwd) {
    process.chdir(originalCwd);
  }

  return result;
}

function mapStreamLines(callback) {
  return new stream.Transform({
    objectMode: true,

    transform(line, _, cb) {
      this.push(callback(line));
      cb();
    }

  });
}
function tapStreamLines(callback) {
  return new stream.Transform({
    objectMode: true,

    transform(line, _, cb) {
      callback(line);
      this.push(line);
      cb();
    }

  });
}
function filterBlankLastLine(blankLine) {
  let bufferedBlankChunk = false;
  return new stream.Transform({
    objectMode: true,

    transform(line, _, callback) {
      if (bufferedBlankChunk) {
        this.push(blankLine);
        bufferedBlankChunk = false;
      }

      if (line === blankLine) {
        bufferedBlankChunk = true;
      } else {
        this.push(line);
      }

      callback();
    }

  });
}

const delay = /*#__PURE__*/util.promisify(setTimeout);
class ServiceProcess {
  constructor(serviceId, serviceConfig, logger, onCrash) {
    this.logTail = [];
    this.didError = false;
    this.didEnd = false;
    this.wasEndCalled = false;
    this.serviceId = serviceId;
    this.serviceConfig = serviceConfig;
    this.logger = logger;
    this.process = spawnProcess(this.serviceConfig);
    this.started = Promise.race([events.once(this.process, 'error'), delay(100)]).then(result => {
      if (result && result[0]) {
        this.didError = true;
        throw result[0];
      }
    });
    this.output = getProcessOutput(this.process);

    if (this.serviceConfig.logTailLength > 0) {
      this.output = this.output.pipe(tapStreamLines(line => {
        this.logTail.push(line);

        if (this.logTail.length > this.serviceConfig.logTailLength) {
          this.logTail.shift();
        }
      }));
    }

    this.ended = Promise.all([this.started.catch(() => {}), events.once(this.output, 'end').then(() => {
      this.didEnd = true;
    })]).then(() => {
      if (!this.didError && !this.wasEndCalled) {
        onCrash();
      }
    });
  }

  isRunning() {
    return !this.didError && !this.didEnd;
  }

  end(windowsCtrlCShutdown) {
    if (!this.wasEndCalled) {
      this.wasEndCalled = true;

      if (this.isRunning()) {
        if (windowsCtrlCShutdown) {
          // ctrl+c signal was already sent to all service processes
          this.forceKillAfterTimeout();
        } else if (process.platform === 'win32') {
          this.process.kill();
        } else {
          this.process.kill('SIGINT');
          this.forceKillAfterTimeout();
        }
      }
    }

    return this.ended;
  }

  forceKillAfterTimeout() {
    if (this.serviceConfig.forceKillTimeout === Infinity) {
      return;
    }

    setTimeout(() => {
      if (this.isRunning()) {
        this.logger.log('info', `Force killing service '${this.serviceId}'`);
        this.process.kill('SIGKILL');
      }
    }, this.serviceConfig.forceKillTimeout);
  }

}

function getProcessOutput(proc) {
  return mergeStream([proc.stdout, proc.stderr].map(stream => stream.setEncoding('utf8')).map(stream$1 => stream.pipeline(stream$1, splitStream(line => `${line}\n`), filterBlankLastLine('\n'), () => {})));
}

const genericMessage = 'This is a bug in composite-service. Please file an issue in https://github.com/zenflow/composite-service/issues';
class InternalError extends Error {
  constructor(message) {
    super(`${message}. ${genericMessage}`);
    Object.setPrototypeOf(this, InternalError.prototype);
  }

}
InternalError.prototype.name = InternalError.name;

const delay$1 = /*#__PURE__*/util.promisify(setTimeout);
class Service {
  constructor(id, config, logger, handleFatalError) {
    this.output = cloneable(new stream.PassThrough({
      objectMode: true
    }));
    this.outputClone = this.output.clone();
    this.crashes = [];
    this.id = id;
    this.config = config;
    this.logger = logger;
    this.handleFatalError = handleFatalError;
  }

  start() {
    if (this.stopResult) {
      this.logger.log('error', new InternalError('Cannot start after stopping').stack);
      return this.startResult;
    }

    if (!this.startResult) {
      this.logger.log('debug', `Starting service '${this.id}'...`);
      this.defineReady();
      this.startResult = this.startProcess().then(() => this.ready).then(() => {
        this.logger.log('debug', `Started service '${this.id}'`);
      });
    }

    return this.startResult;
  }

  defineReady() {
    const ctx = {
      output: this.outputClone
    };
    this.ready = promiseTry(() => this.config.ready(ctx)).finally(() => this.outputClone.destroy()).catch(error => {
      const prefix = `In \`ready\` function for service '${this.id}'`;
      this.handleFatalError(`${prefix}: ${maybeErrorText(error)}`);
      return never();
    });
  }

  async startProcess() {
    const proc = new ServiceProcess(this.id, this.config, this.logger, () => {
      proc.output.unpipe();

      if (!this.stopResult) {
        this.handleCrash(proc);
      }
    });
    this.process = proc;
    proc.output.pipe(this.output, {
      end: false
    });

    try {
      await this.process.started;
    } catch (error) {
      const prefix = `Spawning process for service '${this.id}'`;
      this.handleFatalError(`${prefix}: ${error}`);
      await never();
    }
  }

  async handleCrash(proc) {
    this.logger.log('info', `Service '${this.id}' crashed`);
    const delayPromise = delay$1(this.config.minimumRestartDelay);
    const crash = {
      date: new Date(),
      logTail: proc.logTail
    };
    this.crashes.push(crash);
    const isServiceReady = await isResolved(this.ready);
    const ctx = {
      isServiceReady,
      crash,
      crashes: this.crashes
    };

    try {
      await this.config.onCrash(ctx);
    } catch (error) {
      const prefix = `In \`onCrash\` function for service ${this.id}`;
      this.handleFatalError(`${prefix}: ${maybeErrorText(error)}`);
      await never();
    }

    if (this.stopResult) {
      return;
    }

    await delayPromise;

    if (this.stopResult) {
      return;
    }

    this.logger.log('info', `Restarting service '${this.id}'`);
    await this.startProcess();
  }

  stop(windowsCtrlCShutdown) {
    if (!this.stopResult) {
      if (!this.process || !this.process.isRunning()) {
        this.stopResult = Promise.resolve();
      } else {
        this.logger.log('debug', `Stopping service '${this.id}'...`);
        this.stopResult = this.process.end(windowsCtrlCShutdown).then(() => {
          this.logger.log('debug', `Stopped service '${this.id}'`);
        });
      }
    }

    return this.stopResult;
  }

}
function maybeErrorText(maybeError) {
  return maybeError instanceof Error && maybeError.stack || String(maybeError);
}

function promiseTry(fn) {
  try {
    return Promise.resolve(fn());
  } catch (error) {
    return Promise.reject(error);
  }
}

function isResolved(promise) {
  return Promise.race([promise.then(() => true, () => false), Promise.resolve().then(() => false)]);
}

function never() {
  return new Promise(() => {});
}

function once(fn) {
  var result;
  return function () {
    return result || (result = fn());
  };
}

var typeSuite0 = /*#__PURE__*/once(function () {
  return {
    CompositeServiceConfig: t.iface([], {
      "logLevel": t.opt(t.union(t.lit('debug'), t.lit('info'), t.lit('error'))),
      "gracefulShutdown": t.opt("boolean"),
      "windowsCtrlCShutdown": t.opt("boolean"),
      "serviceDefaults": t.opt("ServiceConfig"),
      "services": t.iface([], {})
    })
  };
});
var typeSuite1 = /*#__PURE__*/once(function () {
  return {
    ServiceConfig: t.iface([], {
      "dependencies": t.opt(t.array("string")),
      "cwd": t.opt("string"),
      "command": t.opt(t.union("string", t.array("string"))),
      "env": t.opt(t.iface([], {})),
      "ready": t.opt(t.func("any", t.param("ctx", "ReadyContext"))),
      "forceKillTimeout": t.opt("number"),
      "onCrash": t.opt(t.func("any", t.param("ctx", "OnCrashContext"))),
      "logTailLength": t.opt("number"),
      "minimumRestartDelay": t.opt("number")
    })
  };
});
function validateAndNormalizeConfig(config) {
  validateType('CompositeServiceConfig', 'config', config);
  const {
    logLevel = 'info'
  } = config;
  const windowsCtrlCShutdown = process.platform === 'win32' && Boolean(config.windowsCtrlCShutdown);
  const gracefulShutdown = !windowsCtrlCShutdown && Boolean(config.gracefulShutdown);
  const {
    serviceDefaults = {}
  } = config;
  doExtraServiceConfigChecks('config.serviceDefaults', serviceDefaults);
  const truthyServiceEntries = Object.entries(config.services).filter(([, value]) => value);

  if (truthyServiceEntries.length === 0) {
    throw new ConfigValidationError('`config.services` has no entries');
  }

  const services = {};

  for (const [id, config] of truthyServiceEntries) {
    services[id] = processServiceConfig(id, config, serviceDefaults);
  }

  validateDependencyTree(services);
  return {
    logLevel,
    gracefulShutdown,
    windowsCtrlCShutdown,
    services
  };
}

function doExtraServiceConfigChecks(path, config) {
  if (typeof config.command !== 'undefined' && (Array.isArray(config.command) ? !config.command.length || !config.command[0].trim() : !config.command.trim())) {
    throw new ConfigValidationError(`\`${path}.command\` has no binary part`);
  }
}

function processServiceConfig(id, config, defaults) {
  const path = `config.services.${id}`;
  validateType('ServiceConfig', path, config);
  doExtraServiceConfigChecks(path, config);
  const merged = {
    dependencies: [],
    cwd: '.',
    // no default command
    env: process.env,
    ready: () => Promise.resolve(),
    forceKillTimeout: 5000,
    onCrash: () => {},
    logTailLength: 0,
    minimumRestartDelay: 1000,
    ...removeUndefinedProperties(defaults),
    ...removeUndefinedProperties(config)
  };

  if (merged.command === undefined) {
    throw new ConfigValidationError(`\`${path}.command\` is not defined`);
  }

  return { ...merged,
    command: normalizeCommand(merged.command),
    env: normalizeEnv(merged.env)
  };
}

function removeUndefinedProperties(object) {
  const result = { ...object
  };

  for (const [key, value] of Object.entries(result)) {
    if (value === undefined) {
      delete result[key];
    }
  }

  return result;
}

function normalizeCommand(command) {
  return Array.isArray(command) ? command : command.split(/\s+/).filter(Boolean);
}

function normalizeEnv(env) {
  return Object.fromEntries(Object.entries(env).filter(([, value]) => value !== undefined).map(([key, value]) => [key, String(value)]));
}

function validateDependencyTree(services) {
  const serviceIds = Object.keys(services);

  for (const [serviceId, {
    dependencies
  }] of Object.entries(services)) {
    for (const dependency of dependencies) {
      if (!serviceIds.includes(dependency)) {
        throw new ConfigValidationError(`Service "${serviceId}" has dependency on unknown service "${dependency}"`);
      }
    }
  }

  for (const serviceId of serviceIds) {
    validateNoCyclicDeps(serviceId, []);
  }

  function validateNoCyclicDeps(serviceId, path) {
    const isLooped = path.includes(serviceId);

    if (isLooped) {
      throw new ConfigValidationError(`Service "${serviceId}" has cyclic dependency ${path.slice(path.indexOf(serviceId)).concat(serviceId).join(' -> ')}`);
    }

    for (const dep of services[serviceId].dependencies) {
      validateNoCyclicDeps(dep, [...path, serviceId]);
    }

    return null;
  }
}

class ConfigValidationError extends Error {
  constructor(message) {
    super(message);
    Object.setPrototypeOf(this, ConfigValidationError.prototype);
  }

}
ConfigValidationError.prototype.name = ConfigValidationError.name;
const checkers = /*#__PURE__*/t.createCheckers({ ... /*#__PURE__*/typeSuite0(),
  ... /*#__PURE__*/typeSuite1()
});

function validateType(typeName, reportedPath, value) {
  const checker = checkers[typeName];
  checker.setReportedPath(reportedPath);
  const error = checker.validate(value);

  if (error) {
    throw new ConfigValidationError(getErrorMessage(error));
  }
}

function getErrorMessage(error) {
  return getErrorMessageLines(error).join('\n');
}

function getErrorMessageLines(error) {
  let result = [`\`${error.path}\` ${error.message}`];

  if (error.nested) {
    for (const nested of error.nested) {
      result = result.concat(getErrorMessageLines(nested).map(s => `    ${s}`));
    }
  }

  return result;
}

const orderedLogLevels = ['error', 'info', 'debug'];
const logLevelColors = {
  error: 'red',
  info: 'teal',
  debug: 'yellow'
};
class Logger {
  constructor(level) {
    this.output = new stream.PassThrough({
      objectMode: true
    });
    this.level = level;
  }

  log(level, text) {
    if (this.shouldLog(level)) {
      const label = chalk.keyword(logLevelColors[level])(`(${level})`);

      for (const line of text.split('\n')) {
        this.output.write(` ${label} ${line}\n`);
      }
    }
  }

  shouldLog(level) {
    return orderedLogLevels.indexOf(level) <= orderedLogLevels.indexOf(this.level);
  }

}

class CompositeService {
  constructor(config) {
    this.stopping = false;
    this.config = validateAndNormalizeConfig(config);

    if (process.platform === 'win32' && this.config.windowsCtrlCShutdown) {
      require('generate-ctrl-c-event'); // make sure this module loads before we even start

    }

    const outputStream = mergeStream();
    outputStream.pipe(process.stdout);
    this.logger = new Logger(this.config.logLevel);
    outputStream.add(this.logger.output);
    const configText = serializeJs(config, {
      space: 2,
      unsafe: true
    });
    this.logger.log('debug', `Config: ${configText}`);
    process.on('SIGINT', () => this.handleShutdownSignal(130, 'SIGINT'));
    process.on('SIGTERM', () => this.handleShutdownSignal(143, 'SIGTERM'));

    if (process.stdin.isTTY) {
      process.stdin.setRawMode(true);
      process.stdin.on('data', buffer => {
        if (buffer.toString('utf8') === '\u0003') {
          this.handleShutdownSignal(130, 'ctrl+c');
        }
      });
    }

    this.services = Object.entries(this.config.services).map(([id, config]) => new Service(id, config, this.logger, this.handleFatalError.bind(this)));
    this.serviceMap = new Map(this.services.map(service => [service.id, service]));
    outputStream.add(this.services.map(({
      output,
      id
    }, i) => {
      // luminosity of 20 because at 256 colors, luminosity from 16 to 24 yields the most colors (12 colors) while keeping high contrast with text
      const label = chalk.bgHsl(i / this.services.length * 360, 100, 20)(id);
      return output.pipe(mapStreamLines(line => `${label} | ${line}`));
    }));
    this.logger.log('debug', 'Starting composite service...');
    Promise.all(this.services.map(service => this.startService(service))).then(() => this.logger.log('debug', 'Started composite service'));
  }

  async startService(service) {
    const dependencies = service.config.dependencies.map(id => this.serviceMap.get(id));
    await Promise.all(dependencies.map(service => this.startService(service)));

    if (this.stopping) {
      await never$1();
    }

    await service.start();

    if (this.stopping) {
      await never$1();
    }
  }

  handleFatalError(message) {
    this.logger.log('error', `Fatal error: ${message}`);

    if (!this.stopping) {
      this.stop(1);
    }
  }

  handleShutdownSignal(exitCode, description) {
    if (!this.stopping) {
      this.logger.log('info', `Received shutdown signal (${description})`);
      this.stop(exitCode);
    }
  }

  stop(exitCode) {
    if (this.stopping) return;
    this.stopping = true;
    this.logger.log('debug', 'Stopping composite service...');

    if (this.config.windowsCtrlCShutdown) {
      require('generate-ctrl-c-event').generateCtrlCAsync().catch(error => this.logger.log('error', String(error)));
    }

    Promise.all(this.services.map(service => this.stopService(service))).then(() => this.logger.log('debug', 'Stopped composite service')) // Wait one micro tick for output to flush
    .then(() => process.exit(exitCode));
  }

  async stopService(service) {
    if (this.config.gracefulShutdown) {
      const dependents = this.services.filter(({
        config
      }) => config.dependencies.includes(service.id));
      await Promise.all(dependents.map(service => this.stopService(service)));
    }

    await service.stop(this.config.windowsCtrlCShutdown);
  }

}

function never$1() {
  return new Promise(() => {});
}

let started = false;
/**
 * Starts a composite service in the current Node.js process
 *
 * @remarks
 *
 * Each service defined in {@link CompositeServiceConfig.services}
 * is started and managed according to its {@link ServiceConfig}.
 *
 * The stdout & stderr from each service is piped to stdout,
 * each line prepended with the service ID.
 *
 * Each service is told to shut down
 * when the composite service is itself told to shut down
 * (with `ctrl+c`, `SIGINT`, or `SIGTERM`),
 * or when a fatal error occurs.
 *
 * All possible fatal errors:
 * (1) A composite service was already started in this process
 * (2) Error validating configuration
 * (3) Error spawning process, e.g. `EPERM`, `ENOENT`, etc.
 * (4) Error from user-provided configuration function,
 * {@link ServiceConfig.ready} or {@link ServiceConfig.onCrash}
 *
 * @param config - Configuration for the composite service
 *
 * @public
 */

function startCompositeService(config) {
  if (started) {
    throw new Error('Already started a composite service in this process');
  }

  new CompositeService(config);
  started = true;
}

/**
 * Waits until `output` emits a `line` for which `test(line)` returns `true`
 *
 * @param output - To scan
 * @param test - To test each `line` from `output`
 *
 * @example
 *
 * ```js
 * const { onceOutputLine } = require('composite-service')
 *
 * const myServiceConfig = {
 *   command: 'node service.js',
 *   ready: ctx => onceOutputLine(ctx.output, line => line.match(/^Ready/)),
 * }
 * ```
 *
 * @public
 */
function onceOutputLine(output, test) {
  return new Promise(resolve => {
    const handler = line => {
      if (test(line)) {
        output.off('data', handler);
        resolve();
      }
    };

    output.on('data', handler);
  });
}
/**
 * Waits until `output` emits a line equal to (`===`) `value`
 *
 * @param output - To scan
 * @param value - To look for in `output`
 *
 * @example
 *
 * ```js
 * const { onceOutputLine } = require('composite-service')
 *
 * const myServiceConfig = {
 *   command: 'node service.js',
 *   ready: ctx => onceOutputLineIs(ctx.output, 'Ready\n'),
 * }
 * ```
 *
 * @public
 */

function onceOutputLineIs(output, value) {
  return onceOutputLine(output, line => line === value);
}
/**
 * Waits until `output` emits a line that `.includes(value)`
 *
 * @param output - To scan
 * @param value - To look for within each line from `output`
 *
 * @example
 *
 * ```js
 * const { onceOutputLineIncludes } = require('composite-service')
 *
 * const myServiceConfig = {
 *   command: 'node service.js',
 *   ready: ctx => onceOutputLineIncludes(ctx.output, 'Ready'),
 * }
 * ```
 *
 * @public
 */

function onceOutputLineIncludes(output, value) {
  return onceOutputLine(output, line => line.includes(value));
}

const delay$2 = /*#__PURE__*/util.promisify(setTimeout);
/**
 * Waits until the given `port` (on the given `host`) is accepting connections
 *
 * @remarks
 *
 * Works by trying to connect to the given port
 * (as opposed to trying to listen on the given port)
 * every 250 milliseconds.
 *
 * @param port -
 * @param host - Defaults to `'localhost'`
 *
 * @example
 *
 * ```js
 * const { onceTcpPortUsed } = require('composite-service')
 *
 * const myServiceConfig = {
 *   command: 'node server.js',
 *   env: { PORT: 3000 },
 *   ready: () => onceTcpPortUsed(3000),
 * }
 * ```
 *
 * @public
 */

async function onceTcpPortUsed(port, host = 'localhost') {
  const portNumber = typeof port === 'number' ? port : parseInt(port, 10);

  while (true) {
    if (await isTcpPortUsed(portNumber, host)) {
      return;
    } else {
      await delay$2(250);
    }
  }
}

function isTcpPortUsed(port, host) {
  return new Promise((resolve, reject) => {
    const socket = new net.Socket();
    socket.once('connect', () => {
      resolve(true);
      cleanUp();
    });
    socket.once('error', error => {
      if (['ECONNREFUSED', 'ETIMEDOUT'].includes(error.code)) {
        resolve(false);
      } else {
        reject(error);
      }

      cleanUp();
    });

    function cleanUp() {
      socket.removeAllListeners();
      socket.end();
      socket.destroy();
      socket.unref();
    }

    socket.connect(port, host);
  });
}

const delay$3 = /*#__PURE__*/util.promisify(setTimeout);
/**
 * Waits a predetermined length of time
 *
 * @param milliseconds - Length of time to wait
 *
 * @example
 *
 * ```js
 * const { onceTimeout } = require('composite-service')
 *
 * const myServiceConfig = {
 *   command: 'node service.js',
 *   ready: () => onceTimeout(1000),
 * }
 * ```
 *
 * @public
 */

async function onceTimeout(milliseconds) {
  await delay$3(milliseconds);
}

/**
 * Generates a {@link ServiceConfig} for an HTTP gateway service
 *
 * @param config - Configuration for the HTTP gateway service
 *
 * @remarks
 *
 * The HTTP gateway service will use an array of
 * [http-proxy-middleware](https://github.com/chimurai/http-proxy-middleware)
 * instances to forward incoming HTTP requests to the appropriate composed (or external)
 * service as determined by the {@link HttpGatewayConfig.proxies | proxies} option.
 *
 * The service can safely be used as a dependency of other services,
 * since its configuration includes {@link ServiceConfig.ready}.
 *
 * @public
 */

function configureHttpGateway(config) {
  const {
    dependencies,
    host,
    port,
    proxies
  } = config;
  return {
    dependencies,
    command: ['node', `${__dirname}/http-gateway-server.js`],
    env: {
      HOST: host || '0.0.0.0',
      PORT: String(port),
      PROXIES: serializeJs(proxies, {
        unsafe: true
      })
    },
    ready: ctx => onceOutputLineIncludes(ctx.output, 'Listening @ http://')
  };
}

exports.configureHttpGateway = configureHttpGateway;
exports.onceOutputLine = onceOutputLine;
exports.onceOutputLineIncludes = onceOutputLineIncludes;
exports.onceOutputLineIs = onceOutputLineIs;
exports.onceTcpPortUsed = onceTcpPortUsed;
exports.onceTimeout = onceTimeout;
exports.startCompositeService = startCompositeService;
//# sourceMappingURL=index.js.map
